/**
 * File:    Motors.cpp
 * Author:  David Zemon
 * Project: MS&T ITRSS... Segway?
 */

#include "Motors.h"

void PWMDualMotorsInit (PWMDualMotors *motors, const uint32_t leftMotorPin,
		const uint32_t rightMotorPin) {
	motors->leftMotor = leftMotorPin;
	motors->rightMotor = rightMotorPin;

	GPIODirModeSet(leftMotorPin | rightMotorPin, GPIO_DIR_OUT);
}

void PWMDualMotorsDrive (const PWMDualMotors *motors, const int16_t left,
		const int16_t right) {
	uint32_t timerLeft, timerRight;

	timerLeft = timerRight = CNT + MOTORS_STATIONARY_PULSE;

	timerLeft += left * MICROSECOND;
	timerRight += right * MICROSECOND;

	// Set both pins high (start the PWM pulse)
	GPIOPinSet(motors->leftMotor | motors->rightMotor);

	// Wait for the first timer to ding
	if (left < right) {
		waitcnt(timerLeft);
		GPIOPinClear(motors->leftMotor);
	} else if (right < left) {
		waitcnt(timerRight);
		GPIOPinClear(motors->rightMotor);
	} else /* Implied: if (timerleft == timerRight) */{
		waitcnt(timerLeft);
		GPIOPinClear(motors->leftMotor | motors->rightMotor);

		return;
	}

	// Wait for the second timer to ding
	if (left > right) {
		waitcnt(timerLeft);
		GPIOPinClear(motors->leftMotor);
	} else {
		waitcnt(timerRight);
		GPIOPinClear(motors->rightMotor);
	}
}
