/* File:    Segway_backend.h
 * 
 * Author:  David Zemon
 * Project: Segway
 */

#ifndef SEGWAY_BACKED_H_
#define SEGWAY_BACKED_H_

#include <l3g.h>
#include <mcp300x.h>
#include "Segway_globals.h"
#include <hd44780.h>

#define LCD_DATA_MASK			BIT_26 | BIT_25 | BIT_24 | BIT_23
#define LCD_RS					BIT_10
#define LCD_RW					BIT_27
#define LCD_EN					BIT_8
#define LCD_BITMODE				HD44780_4BIT
#define LCD_DIM					HD44780_16x2

// SPI Init Values
#define MOSI					BIT_1
#define MISO					BIT_0
#define SCLK					BIT_2
#define SPI_FREQ				100000
#define ADC_SPI_MODE			MCP300X_SPI_MODE
#define ADC_SPI_BITMODE			MCP300X_SPI_BITMODE
#define ADC_CS					BIT_4
#define GYRO_SPI_MODE			L3G_SPI_MODE
#define GYRO_SPI_BITMODE		L3G_SPI_BITMODE
#define GYRO_CS					BIT_8

// Analog related constants
#define ACCEL_CENTER			522						// Change to calibrate center of balance
#define	TURN_POT				MCP_CHANNEL_3			// potentiometer input connected to the hinge of the handlebar
#define	ACCEL 					MCP_CHANNEL_1			// accelerometer input
// Do this thing...
#define TURN_CENTER				512					// Change to calibrate center of turn potentiometer
// Tuning Variables
#define LEAN_TO_TURN_READ_RATIO	4					// Read in the accelerometer and gyro values X times more than the turn pot
#define GYRO_WEIGHT				0.8
#define FILTER_TIME_CONSTANT	0.5f				// Adjusts the time constant of the filter; must be between 0 and 1
#define TURN_NULL_ZONE			5					// Range of non-response for the turn angle from zero in degrees
#define MAX_TURN_POWER			20					// Max allowable angle reading for the turn potentiometer
// Filtering constants and type definitions
#define GYRO_ARRAY_SIZE			32
#define ACCEL_ARRAY_SIZE		32
#define ACCEL_WEIGHT_FUNCTION	((ACCEL_ARRAY_SIZE - j) * 100UL / ACCEL_ARRAY_SIZE)
#define GYRO_WEIGHT_FUNCTION	((GYRO_ARRAY_SIZE - j) * 100UL / GYRO_ARRAY_SIZE)

typedef enum {
	ACCELEROMETER,
	GYROSCOPE,
	NUM_FILTERS
} filter_types_t;

typedef uint16_t filter_size_t; // Filter sizes may change to required 8- or 16-bit variables

typedef struct {
		int16_t *valueArray;
		uint8_t *weightArray;
		uint8_t head;
		uint8_t tail;
		filter_size_t size;
} filter_t;

/*****************************
 *** Function Declarations ***
 *****************************/
void segway_backend (void *par);

static void backend_setup (filter_t filters[], int16_t *filterValues[],
		uint8_t *filterWeights[]);

static int8_t readEssentials (uint32_t *prevTimeCnt, filter_t filters[]);

static int8_t readNonEssentials (void);

static int16_t runFilter (filter_t filter, int16_t newVal);

#endif /* SEGWAY_BACKED_H_ */
